@@ -3,7 +3,8 @@ import io
 import os
 import sys
 import tempfile
-from typing import Optional
+from typing import Optional, Tuple, Type, Union
+from unittest import mock
 
 import pytest
 
@@ -122,6 +123,48 @@ def test_size():
     assert w == 99 and h == 101
 
 
+@pytest.mark.parametrize(
+    "is_windows,no_descriptor_size,stdin_size,stdout_size,stderr_size,expected_size",
+    [
+        # on Windows we'll use `os.get_terminal_size()` without arguments...
+        (True, (133, 24), ValueError, ValueError, ValueError, (133, 24)),
+        (False, (133, 24), ValueError, ValueError, ValueError, (80, 25)),
+        # ...while on other OS we'll try to pass stdin, then stdout, then stderr to it:
+        (False, ValueError, (133, 24), ValueError, ValueError, (133, 24)),
+        (False, ValueError, ValueError, (133, 24), ValueError, (133, 24)),
+        (False, ValueError, ValueError, ValueError, (133, 24), (133, 24)),
+        (False, ValueError, ValueError, ValueError, ValueError, (80, 25)),
+    ],
+)
+@mock.patch("rich.console.os.get_terminal_size")
+def test_size_can_fall_back_to_std_descriptors(
+    get_terminal_size_mock: mock.MagicMock,
+    is_windows: bool,
+    no_descriptor_size: Union[Tuple[int, int], Type[ValueError]],
+    stdin_size: Union[Tuple[int, int], Type[ValueError]],
+    stdout_size: Union[Tuple[int, int], Type[ValueError]],
+    stderr_size: Union[Tuple[int, int], Type[ValueError]],
+    expected_size,
+):
+    def get_terminal_size_mock_impl(fileno: int = None) -> Tuple[int, int]:
+        value = {
+            None: no_descriptor_size,
+            sys.__stdin__.fileno(): stdin_size,
+            sys.__stdout__.fileno(): stdout_size,
+            sys.__stderr__.fileno(): stderr_size,
+        }[fileno]
+        if value is ValueError:
+            raise value
+        return value
+
+    get_terminal_size_mock.side_effect = get_terminal_size_mock_impl
+
+    console = Console(legacy_windows=False)
+    with mock.patch("rich.console.WINDOWS", new=is_windows):
+        w, h = console.size
+    assert (w, h) == expected_size
+
+
 def test_repr():
     console = Console()
     assert isinstance(repr(console), str)
@@ -450,6 +493,158 @@ def test_export_html_inline():
     assert html == expected
 
 
+EXPECTED_SVG = """\
+<svg width="1484.0" height="384" viewBox="0 0 1484.0 384"
+     xmlns="http://www.w3.org/2000/svg">
+    <style>
+        @font-face {
+            font-family: "Rich Fira Code";
+            src: local("FiraCode-Regular"),
+                 url("https://cdnjs.cloudflare.com/ajax/libs/firacode/6.2.0/woff2/FiraCode-Regular.woff2") format("woff2"),
+                 url("https://cdnjs.cloudflare.com/ajax/libs/firacode/6.2.0/woff/FiraCode-Regular.woff") format("woff");
+            font-style: normal;
+            font-weight: 400;
+        }
+        @font-face {
+            font-family: "Rich Fira Code";
+            src: local("FiraCode-Bold"),
+                 url("https://cdnjs.cloudflare.com/ajax/libs/firacode/6.2.0/woff2/FiraCode-Bold.woff2") format("woff2"),
+                 url("https://cdnjs.cloudflare.com/ajax/libs/firacode/6.2.0/woff/FiraCode-Bold.woff") format("woff");
+            font-style: bold;
+            font-weight: 700;
+        }
+        .rich-svg-${SVG_HASH}-terminal-wrapper span {
+            display: inline-block;
+            white-space: pre;
+            vertical-align: top;
+            font-size: 18px;
+            font-family:'Rich Fira Code','Cascadia Code',Monaco,Menlo,'DejaVu Sans Mono',consolas,'Courier New',monospace;
+        }
+        .rich-svg-${SVG_HASH}-terminal-wrapper a {
+            text-decoration: none;
+            color: inherit;
+        }
+        .rich-svg-${SVG_HASH}-terminal-body .blink {
+           animation: rich-svg-${SVG_HASH}-blinker 1s infinite;
+        }
+        @keyframes rich-svg-${SVG_HASH}-blinker {
+            from { opacity: 1.0; }
+            50% { opacity: 0.3; }
+            to { opacity: 1.0; }
+        }
+        .rich-svg-${SVG_HASH}-terminal-wrapper {
+            padding: 140px;
+            padding-top: 100px;
+        }
+        .rich-svg-${SVG_HASH}-terminal {
+            position: relative;
+            display: flex;
+            flex-direction: column;
+            align-items: center;
+            background-color: #0c0c0c;
+            border-radius: 14px;
+            box-shadow: 0 0 0 1px #484848;
+        }
+        .rich-svg-${SVG_HASH}-terminal:after {
+            position: absolute;
+            width: 100%;
+            height: 100%;
+            content: '';
+            border-radius: 14px;
+            background: rgb(71,77,102);
+            background: linear-gradient(90deg, #804D69 0%, #4E4B89 100%);
+            transform: rotate(-4.5deg);
+            z-index: -1;
+        }
+        .rich-svg-${SVG_HASH}-terminal-header {
+            position: relative;
+            width: 100%;
+            background-color: #2e2e2e;
+            margin-bottom: 12px;
+            font-weight: bold;
+            border-radius: 14px 14px 0 0;
+            color: #f2f2f2;
+            font-size: 18px;
+            box-shadow: inset 0px -1px 0px 0px #4e4e4e,
+                        inset 0px -4px 8px 0px #1a1a1a;
+        }
+        .rich-svg-${SVG_HASH}-terminal-title-tab {
+            display: inline-block;
+            margin-top: 14px;
+            margin-left: 124px;
+            font-family: sans-serif;
+            padding: 14px 28px;
+            border-radius: 6px 6px 0 0;
+            background-color: #0c0c0c;
+            box-shadow: inset 0px 1px 0px 0px #4e4e4e,
+                        0px -4px 4px 0px #1e1e1e,
+                        inset 1px 0px 0px 0px #4e4e4e,
+                        inset -1px 0px 0px 0px #4e4e4e;
+        }
+        .rich-svg-${SVG_HASH}-terminal-traffic-lights {
+            position: absolute;
+            top: 24px;
+            left: 20px;
+        }
+        .rich-svg-${SVG_HASH}-terminal-body {
+            line-height: 22px;
+            padding: 14px;
+        }
+        .rich-svg-${SVG_HASH}-terminal-body .r1 {color: #f2f2f2; text-decoration-color: #f2f2f2;background-color: #0c0c0c;}
+.rich-svg-${SVG_HASH}-terminal-body .r2 {color: #2472c8; text-decoration-color: #2472c8; background-color: #cd3131; font-weight: bold}
+.rich-svg-${SVG_HASH}-terminal-body .r3 {;color: #f2f2f2; text-decoration-color: #f2f2f2;;background-color: #0c0c0c;}
+    </style>
+    <foreignObject x="0" y="0" width="100%" height="100%">
+        <body xmlns="http://www.w3.org/1999/xhtml">
+            <div class="rich-svg-${SVG_HASH}-terminal-wrapper">
+                <div class="rich-svg-${SVG_HASH}-terminal">
+                    <div class="rich-svg-${SVG_HASH}-terminal-header">
+                        <svg class="rich-svg-${SVG_HASH}-terminal-traffic-lights" width="90" height="21" viewBox="0 0 90 21" xmlns="http://www.w3.org/2000/svg">
+                            <circle cx="14" cy="8" r="8" fill="#ff6159"/>
+                            <circle cx="38" cy="8" r="8" fill="#ffbd2e"/>
+                            <circle cx="62" cy="8" r="8" fill="#28c941"/>
+                        </svg>
+                        <div class="rich-svg-${SVG_HASH}-terminal-title-tab">Rich</div>
+                    </div>
+                    <div class="rich-svg-${SVG_HASH}-terminal-body">
+                        <div><span class="r2">foo</span><span class="r1"> </span><span class="r3"><span class="blink"><a href="https://example.org">Click</a></span></span><span class="r1">                                                                                           </span></div>
+<div><span class="r1"></span><span class="r1">                                                                                                    </span></div>
+                    </div>
+                </div>
+            </div>
+        </body>
+    </foreignObject>
+</svg>
+"""
+
+
+def test_export_svg():
+    console = Console(record=True, width=100)
+    console.print(
+        "[b red on blue reverse]foo[/] [blink][link=https://example.org]Click[/link]"
+    )
+    svg = console.export_svg()
+    svg_main_code_hash = (
+        "857433718"  # hard-coded here after the 1st time we ran this test
+    )
+    expected_svg = EXPECTED_SVG.replace("${SVG_HASH}", svg_main_code_hash)
+    assert svg == expected_svg
+
+
+def test_save_svg():
+    console = Console(record=True, width=100)
+    console.print(
+        "[b red on blue reverse]foo[/] [blink][link=https://example.org]Click[/link]"
+    )
+    svg_main_code_hash = "857433718"
+    expected_svg = EXPECTED_SVG.replace("${SVG_HASH}", svg_main_code_hash)
+    with tempfile.TemporaryDirectory() as path:
+        export_path = os.path.join(path, "example.svg")
+        console.save_svg(export_path)
+        with open(export_path, "rt") as svg_file:
+            assert svg_file.read() == expected_svg
+
+
 def test_save_text():
     console = Console(record=True, width=100)
     console.print("foo")
@@ -682,6 +877,18 @@ def test_is_alt_screen():
     assert not console.is_alt_screen
 
 
+def test_set_console_title():
+    console = Console(force_terminal=True, _environ={})
+    if console.legacy_windows:
+        return
+
+    with console.capture() as captured:
+        console.set_window_title("hello")
+
+    result = captured.get()
+    assert result == "\x1b]0;hello\x07"
+
+
 def test_update_screen():
     console = Console(force_terminal=True, width=20, height=5, _environ={})
     if console.legacy_windows:
